'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

exports.__esModule = true;
exports.subtree = subtree;
exports.simple = simple;
exports.sequence = sequence;

var _isArray$isObject$mapValues = require('./utils');

function compile(expr) {
  if (typeof expr === 'object') {
    return function (node, context) {
      for (var key in expr) {
        if (!expr[key](node[key], context)) {
          return false;
        }
      }

      return true;
    };
  }

  return function (node, context) {
    return expr(node, context);
  };
}

function subtree(node) {
  return node != null;
}

function simple(node) {
  return subtree(node) && !_isArray$isObject$mapValues.isArray(node) && !_isArray$isObject$mapValues.isObject(node);
}

function sequence(node) {
  return subtree(node) && _isArray$isObject$mapValues.isArray(node) && node.every(simple);
}

var Transformer = (function () {
  function Transformer() {
    var rules = arguments[0] === undefined ? [] : arguments[0];

    _classCallCheck(this, Transformer);

    this.rules = rules.map(function (_ref) {
      var match = _ref.match;
      var transform = _ref.transform;
      return { match: compile(match), transform: transform };
    });
  }

  Transformer.prototype.rule = function rule(match, transform) {
    this.rules.push({ match: compile(match), transform: transform });
    return this;
  };

  Transformer.prototype.run = function run(node, context) {
    var _this = this;

    var runInContext = function runInContext(child) {
      return _this.run(child, context);
    };

    if (_isArray$isObject$mapValues.isArray(node)) {
      node = node.map(runInContext);
    } else if (_isArray$isObject$mapValues.isObject(node)) {
      node = _isArray$isObject$mapValues.mapValues(node, runInContext);
    }

    for (var i = 0; i < this.rules.length; i++) {
      var _rules$i = this.rules[i];
      var match = _rules$i.match;
      var transform = _rules$i.transform;

      if (match(node, context)) {
        return transform(node, context);
      }
    }

    return node;
  };

  _createClass(Transformer, null, [{
    key: 'subtree',
    value: subtree,
    enumerable: true
  }, {
    key: 'simple',
    value: simple,
    enumerable: true
  }, {
    key: 'sequence',
    value: sequence,
    enumerable: true
  }]);

  return Transformer;
})();

exports['default'] = Transformer;